package cn.ibizlab.util.aspect;

import com.alibaba.fastjson.JSON;
import com.alibaba.rocketmq.client.producer.DefaultMQProducer;
import com.alibaba.rocketmq.common.message.Message;
import lombok.extern.slf4j.Slf4j;
import cn.ibizlab.util.domain.EntityBase;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Lazy;
import org.springframework.expression.EvaluationContext;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import java.util.List;

/**
 * rocketMQ消息切面
 */
@Aspect
@Component
@Slf4j
public class RocketMQAspect
{
    private final ExpressionParser parser = new SpelExpressionParser();
    @Autowired
    @Qualifier("SysEmployeedeleteSysUserproducer")
    @Lazy
    DefaultMQProducer SysEmployeedeleteSysUserproducer;

    @AfterReturning(value = "(execution(* cn.ibizlab.core.*.service.*SysEmployee*.create*(..))||execution(* cn.ibizlab.core.*.service.*SysEmployee*.update*(..))||execution(* cn.ibizlab.core.*.service.*SysEmployee*.save*(..)) ||execution(* cn.ibizlab.core.*.service.*SysEmployee*.remove*(..))) && !execution(* cn.ibizlab.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.core.es.service.*.save*(..)) && !execution(* cn.ibizlab.core.es.service.*.remove*(..))")
    @Async
    public void sysEmployeedeleteSysUserproducer(JoinPoint point) {
        sendMsg(SysEmployeedeleteSysUserproducer, "deleteSysUser", "deleteSysUser", getEntity(point));
    }
    @Autowired
    @Qualifier("SysEmployeesaveSysUserproducer")
    @Lazy
    DefaultMQProducer SysEmployeesaveSysUserproducer;

    @AfterReturning(value = "(execution(* cn.ibizlab.core.*.service.*SysEmployee*.create*(..))||execution(* cn.ibizlab.core.*.service.*SysEmployee*.update*(..))||execution(* cn.ibizlab.core.*.service.*SysEmployee*.save*(..)) ||execution(* cn.ibizlab.core.*.service.*SysEmployee*.remove*(..))) && !execution(* cn.ibizlab.core.es.service.*.create*(..)) && !execution(* cn.ibizlab.core.es.service.*.update*(..)) && !execution(* cn.ibizlab.core.es.service.*.save*(..)) && !execution(* cn.ibizlab.core.es.service.*.remove*(..))")
    @Async
    public void sysEmployeesaveSysUserproducer(JoinPoint point) {
        sendMsg(SysEmployeesaveSysUserproducer, "saveSysUser", "saveSysUser", getEntity(point));
    }

    /**
     * 输出过滤行为
     * @param point
     * @param actionName
     */
    private void outputAction(JoinPoint point,String actionName) {
        Object [] args = point.getArgs();
        if(ObjectUtils.isEmpty(args) || args.length==0) {
            return;
        }
        Object arg = args[0];
        Object service = point.getTarget();
        EvaluationContext serviceCtx = new StandardEvaluationContext();
        serviceCtx.setVariable("service", service);
        serviceCtx.setVariable("arg", arg);
        Expression serviceExp = parser.parseExpression(String.format("#service.%s(#arg)", actionName));
        serviceExp.getValue(serviceCtx);
    }

    /**
     * 获取实体对象
     * @param point
     * @return
     */
    private Object getEntity(JoinPoint point) {
        Object entity=null;
        String action=point.getSignature().getName();
        Object [] args = point.getArgs();
        if(ObjectUtils.isEmpty(args) || args.length==0 || StringUtils.isEmpty(action)) {
            return entity;
        }
        Object arg=args[0];
        if(arg instanceof EntityBase  || arg instanceof List) {
            return arg;
        }
        else {
            return null;
        }
    }

    /**
     * 发送消息
     * @param producer
     * @param topic
     * @param tag
     * @param body
     */
    private void sendMsg(DefaultMQProducer producer, String topic, String tag, Object body) {
        if(ObjectUtils.isEmpty(body)) {
            log.error("发送消息失败，无法获取到要发送的消息内容！");
            return;
        }
        try {
            producer.start();
            Message message = new Message(topic, tag, JSON.toJSONString(body).getBytes());
            producer.send(message);
            producer.shutdown();
        } catch (Exception e) {
            log.error("消息发送异常，"+e);
        }
    }
}
